// NeoPixel ColorTuner 2, using Adafruit ILI3941, NeoPixel libraries
//   by Zonker Harris, 2017-03-18
// Based on my RGB NeoPixel ColorTuner project v.7 code
//  There are now 5 pots (R, G, B, W, and brightness), and a rotary encoder
//  The display is now an ILI3941 color OLED display, and I have 5 pixels inside.
// I like some of my older colors, so I'm adding some back into the array!
// 
//  v2: The display blanked too fast in v1, and color pixels were off-by-one

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
Wiring the circuit:
(using an Adafruit Trinket 3.3v! Any 5v signals will DAMAGE the display logic!)
 * A0 is open...
 * Wiper from RED potentiometer to A1
 * Wiper from GREEN potentiometer to A2
 * Wiper from BLUE potentiometer to A3
 * Wiper from WHITE potentiometer to A4
 * Wiper from Brightness potentiometer to A5
 *** digital pin 2 is used by USB Bootloader on the Trinket Pro...
     https://learn.adafruit.com/introducing-pro-trinket/pinouts 
 * Rotary Encoder output A to digital pin 3
 * Rotary Encoder output B to digital pin 4
 * Rotary Encoder bushbutton output to digital pin 5
 * NeoPixel data output to pin 6
 *** digital pin 7 is used by USB Bootloader on the Trinket Pro...
 * Display RESET to digital pin 8
 * Display D/C to digital pin 9
 * Display CS to digital pin 10
 * Display MOSI to digital pin 11
 * Display MISO to digital pin 12
 * Display SCLK to digital pin 13
 * Display LED can be switched, from +Vbat, through a 100-150 ohm resistor.
 * Display GND to GND
 * Display VCC to +Vbat! (It won't work with 3.3v power!)
 * 
    IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
    pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
    and minimize distance between Arduino and first pixel.  Avoid connecting
    on a live circuit...if you must, connect GND first.
*/

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
ILI3941 Display Clues
  I got my displays from PJRC, along with some Teensy3.x boards. Clues:
   https://www.pjrc.com/store/display_ili9341.html

 Caution: Use only 3.3V logic signals with this display. 
   The CS, D/C, MOSI, MISO, SCK pins  are *NOT* 5 volt tolerant. 
   We recommend using this display only with Teensy LC & 3.x. 
   It should not be used with 5V boards like Teensy 2.0 and Arduino Uno. 
   
  We do not recommend using the SD card socket on this display. 
  It does not work, but this hardware modification may get it working. 
   https://forum.pjrc.com/threads/28106-Display_ili9341?p=72889&viewfull=1#post72889 

  If you plan to use the display with the Teensy, you'll need to read more
   at the URL above, especially if you plan to use the Adafruit library.
   
  J2 has an index on the silkscreen! The pin with the square is VCC!
    VCC    (This should be 3.6v-5.5v! 3.3v is too low!
            I'm using the VBAT+ from the Trinket Pro 3.3v.)
    GND   (This is the common reference for all signals.)
    CS    
    RESET (tie to 3.3v, you don't need a pin to control this)
    D/C   
    SDI/MOSI (this is TFT_MOSI, an input on the display)
    SCK    (this is the SDI Clock from the Arduino into the display)
    LED    (This powers the backlight of the display...
            I'm using the VBAT+ on the Trinket Pro 3.3v
            with a 100 ohm resistor to LED on the ILI3941)
    SDO/MISO (this is TFT_MISO, an output from the display)

  The variable "rotation" (0-3) sets which side of the screen is the top...
    0 = J2 is at the bottom of the screen (portrait mode)
    1 = J2 is at the right-hand edge (landscape)
    2 = J2 is at the top of the screen (portrait mode)
    3 = J2 is at the left-hand edge (landscape)

  Font size 1 is really tiny! Only 7 pixels tall!
    (39 portrait, 52 landscape)
  Font size 2 is 12 pixels tall, 2 pixel stroke.
    (19 in portrait, 26 landscape)
  Font size 3 is 17 pixels tall, 3 pixel stroke
    (12 in portrait, 15 landscape)
  Font size 4 is pixels tall
    (only 9 in portrait! 12 in landscape)
  Font size 5 is 28 pixels tall, 4 pixel stroke
    (only 7 in portrait! 10 in landscape)

  Colors found in the Adafruit_ILI9341.h file: syntax  ILI9341_BLACK
    BLACK, NAVY, DARKGREEN, DARKCYAN, MAROON, PURPLE, OLIVE, LIGHTGREY
    DARKGREY, BLUE, GREEN, CYAN, RED, MAGENTA, YELLOW, WHITE, ORANGE,
    GREENYELLOW, PINK
  (Colors seem to be defined by a 16-bit hex code, white is 0xFFFF)
  (You should be able to add custom colors to the table...)
  - - - - - - - */

#include "SPI.h"
#include "Adafruit_ILI9341.h"
#include "Adafruit_GFX.h" 

// For the Adafruit Trinket Pro:
#define TFT_DC 9     // Trinket Pro-out to TFT
#define TFT_CS 10    // Trinket Pro-out to TFT
#define TFT_MOSI 11  // Trinket Pro-IN from TFT
#define TFT_CLK 13   // Trinket Pro-out to TFT
#define TFT_RST 8    // Trinket Pro-out to TFT
#define TFT_MISO 12  // Trinket Pro-out to TFT

// Use hardware SPI (on Uno, #13, #12, #11) and the above for CS/DC
//Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
// If using the breakout, change pins as desired
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_MOSI, TFT_CLK, TFT_RST, TFT_MISO);

uint8_t rotation=0;  // declare it here, to use it as a Global...

#include <Adafruit_NeoPixel.h>
#define NumLEDs 64 // how many pixels are used in the string, plus 5 for the console!
#define PixelPin 6
// There will be at least *5* pixels in the box, as the last pixels!
// In order, Red, Green, Blue and the RGB-Mix will be on the panel
// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
// NEO_RGB Pixels are wired for RGB bitstream
// NEO_GRB Pixels are wired for GRB bitstream
// NEO_KHZ400 400 KHz bitstream (e.g. FLORA pixels)
// NEO_KHZ800 800 KHz bitstream (e.g. High Density LED strip)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NumLEDs, PixelPin, NEO_GRBW + NEO_KHZ800);

int aRedPin = A1;    // Analog input pin, red potentiometer input
int aGreenPin = A2;  // Analog input pin, green potentiometer input
int aBluePin = A3;   // Analog input pin, blue potentiometer input
int aWhitePin = A4;  // Analog input pin, blue potentiometer input
int aBritePin = A5;  // Analog input pin, blue potentiometer input

#define EncAPin 3   // Rotary Encoder output A
#define EncBPin 4   // Rotary Encoder output B
#define EncSPin 5   // Rotary Encoder pushbutton output
#define pixelPin 6  // Output to the neopixel strip...

boolean lastButton = LOW;
boolean currentButton = LOW;
boolean ledOn = false;

int colorMax = 19;   //  How many color groups do we have?
// Our "numPixels" variable should be > 2x colorMax plus 5...

int redValue = 0;
int greenValue = 0;
int blueValue = 0;
int whiteValue = 0;
int briteValue = 0;
 
int LEDChan[NumLEDs][4] = {0};
  
int menuMode = 0;  // 0=color tuning, other modes play an animation
int myPrevious = 0;

/* Prepare the colors array
 myBase is modulo 15, with R-G-B values, from brightest to dimmest
 0=white, 15=red, 30=yellow, 45=green, 60=blue, 75=teal, 90=purple
 value 135=black (all zeroes).
 myColorCode is modulo 3, defining one shade of a given color in R-G-B.
 You can use any modulo-3 number in the array, from 0 (white) to 135 (black).
 18=almost-brightest red, 51 is medium-green, 72 is dimmest-blue. */
//Modified for NeoPixels…
int colorSets = 19;
int myColors[] = {
  //(0, 0) brite yellow
  160, 171, 0, 0,
  37, 45, 0, 0,
  //(1, 6)  blue
  0, 0, 160, 0,
  0, 0, 60, 0,
  //(2, 12)  mango, dark orange
  145, 25, 0, 0,
  75, 12, 0, 0,
  //(3, 18)  emerald green
  0, 160, 0, 0,
  0, 60, 0, 0,
  //(4, 24)  dark pink
  135, 0, 26, 0,
  45, 0, 10, 0,
  //(5, 30)  salmon
  112, 24, 20, 0,
  43, 12, 8, 0,
  //(6, 36)  bright pink
  134, 0, 126, 0,
  59, 0, 41, 0,
  //(7, 42)  red
  160, 0, 0, 0,
  60, 0, 0, 0,
  //(8, 48)  orange yellow
  137, 104, 0, 0,
  50, 44, 0, 0,
  //(9, 54)  brite green-yellow
  110, 152, 0, 0,
  59, 84, 0, 0,
  //(10, 60)  dark purple
  62, 0, 114, 0,
  31, 0, 56, 0,
  //(11, 66)  fuschia
  180, 0, 157, 0,
  53, 0, 42, 0,
  //(12, 72)  light lime
  49, 154, 0, 0,
  30, 75, 0, 0,
  //(13, 78)  cyan
  0, 90, 117, 0,
  0, 33, 45, 0,
  //(14, 84)  sky blue
  28, 162, 112, 0,
  22, 62, 49, 0,
  //(15, 90)  teal
  0, 167, 43, 0,
  0, 70, 20, 0,
  //(16, 96)  orange
  160, 60, 0, 0,
  66, 28, 0, 0,
  //(17, 102)  light green
  43, 175, 10, 0,
  22, 64, 8, 0,
  //(18, 108)  lavender
  146, 104, 182, 0,
  89, 71, 106, 0,
  //(19, 114) black
  0, 0, 0, 0};

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void setup() {
  
  // darken the LED strip
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  // Set up our inputs and outputs...
  pinMode(aRedPin, INPUT);    // A1  wiper of a  10-k pot between 5v and ground
  pinMode(aGreenPin, INPUT);  // A2    "
  pinMode(aBluePin, INPUT);   // A3    "
  pinMode(aWhitePin, INPUT);  // A4    "
  pinMode(aBritePin, INPUT);  // A5    "
  // digital pin 2 is used for USB Bootloader on Adafruit Trinket Pro 3v
  pinMode(EncAPin, INPUT);    // digital pin 3
  pinMode(EncBPin, INPUT);    // digital pin 4
  pinMode(EncSPin, INPUT);    // digital pin 5
  pinMode(pixelPin, OUTPUT);  // digital pin 6
  // digital pin 7 is used for USB Bootloader on Adafruit Trinket Pro 3v
  pinMode(TFT_RST, OUTPUT);   // digital pin 8
  pinMode(TFT_DC, OUTPUT);    // digital pin 9
  pinMode(TFT_CS, OUTPUT);    // digital pin 10
  pinMode(TFT_MOSI, INPUT);   // digital pin 11
  pinMode(TFT_MISO, OUTPUT);  // digital pin 12
  pinMode(TFT_CLK, OUTPUT);   // digital pin 13

  digitalWrite(EncSPin, HIGH);  // ensure we have a pull-up, in case of an open

  //paint the display black, and splash some text... 
  tft.begin();
  tft.fillScreen(ILI9341_BLACK);
  tft.setCursor(0, 0);
  tft.setTextColor(ILI9341_WHITE);   tft.setTextSize(2);
  tft.println("Color Tuner 2");
  tft.println("Zonker Harris, 2018");
  tft.setTextSize(1);
  tft.println("");
  tft.setTextColor(ILI9341_YELLOW);  tft.setTextSize(2);
  tft.println("ver 0.2");
  tft.setTextColor(ILI9341_GREEN);
  tft.println("Display Test");
  tft.setTextSize(1);
  tft.println("");
  tft.setTextColor(ILI9341_CYAN);  tft.setTextSize(2);
  tft.println("CPU: Trinket Pro 3v");
  tft.println("Display: ILI9341");
  tft.println("Initializing...");
  delay (200);

  Serial.begin(9600);  // Pseudo-serial for Monitor 9600bps
  Serial.println("ILI9341 Test!"); 

  // Write the current color library to the strip
  int libIndex = 0;
  for (int pixel = 6; pixel < (6 + (2 * colorSets)); pixel++)  
  {
    int red   = myColors[libIndex];
    int green = myColors[libIndex + 1];
    int blue  = myColors[libIndex + 2];
    int white = myColors[libIndex + 3];
    setPixel(strip.Color(red, green, blue, white), pixel, 5);
    libIndex = libIndex + 4;
  }
  strip.show();
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
boolean debounce(boolean last)
  {
  boolean current = digitalRead (EncSPin);
  if (last != current)
  { 
    delay(5);
    current = digitalRead(EncSPin);
  }
  return current;
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
void loop() {
  // Read in the three analog values 
  redValue = (analogRead (aRedPin)/4 -10);
  greenValue = (analogRead (aGreenPin)/4 -10);
  blueValue = (analogRead (aBluePin)/4 -10);
  whiteValue = (analogRead (aWhitePin)/4 -10);
  briteValue = (analogRead (aBritePin)/4 -10);
  if (redValue < 0) {redValue = 0;}
  if (greenValue < 0) {greenValue = 0;}
  if (blueValue < 0) {blueValue = 0;}
  if (whiteValue < 0) {whiteValue = 0;}
  if (briteValue < 0) {briteValue = 0;}
  setPixel(strip.Color(redValue, 0, 0, 0), 4, 0); 
  setPixel(strip.Color(0, greenValue, 0, 0), 3, 0); 
  setPixel(strip.Color(0, 0, blueValue, 0), 2, 0); 
  setPixel(strip.Color(0, 0, 0,  whiteValue), 1, 0); 
  setPixel(strip.Color(redValue, greenValue, blueValue, whiteValue), 0, 0); 

  Serial.println("-- Top of the main loop");
  Serial.print("red: "); Serial.println(redValue);
  Serial.print("green: "); Serial.println(greenValue);
  Serial.print("blue: "); Serial.println(blueValue);
  Serial.print("white: "); Serial.println(whiteValue);
  Serial.print("brite: "); Serial.println(briteValue);
  //We need to begin the TFT in the main loop, it appears... 
  // Colors found in the Adafruit_ILI9341.h file: syntax  ILI9341_BLACK
  // BLACK, NAVY, DARKGREEN, DARKCYAN, MAROON, PURPLE, OLIVE, LIGHTGREY
  // DARKGREY, BLUE, GREEN, CYAN, RED, MAGENTA, YELLOW, WHITE, ORANGE,
  // GREENYELLOW, PINK
  // Start by writing rectangles of black, to cover "old" data...
  //  (writing blank spaces does NOT replace the characters previously in place!)
  //  (writing many small rectangles is faster than re-drawing the entire screen)
  uint16_t bgcolor = 0x0000;  // set this to 0x0972 if you want to see the rectangles
  tft.fillRect(0, 112, 320, 16, bgcolor);  // cover the "Initializing..." line from startup
  tft.fillRect(90, 140, 55, 24, bgcolor);  // RED
  tft.fillRect(125, 162, 55, 24, bgcolor);  // GREEN
  tft.fillRect(105, 186, 55, 24, bgcolor);  // BLUE
  tft.fillRect(120, 210, 55, 24, bgcolor);  // WHITE
  tft.fillRect(120, 234, 55, 24, bgcolor);  // BRIGHT
  // now add the real data, starting below the header we print in startup...
  tft.setCursor(0, 140);  // Horiz, vert pixels from "upper-left" 
  tft.setTextColor(ILI9341_RED);   tft.setTextSize(3);
  tft.print("RED: ");
  tft.print(redValue);
  tft.println("  ");
  tft.setTextColor(ILI9341_DARKGREEN);   tft.setTextSize(3);
  tft.print("GREEN: ");
  tft.print(greenValue);
  tft.println("  ");
  tft.setTextColor(ILI9341_BLUE);   tft.setTextSize(3);
  tft.print("BLUE: ");
  tft.print(blueValue);
  tft.println("  ");
  tft.setTextColor(ILI9341_WHITE);   tft.setTextSize(3);
  tft.print("WHITE: ");
  tft.print(whiteValue);
  tft.println("  ");
  tft.setTextColor(ILI9341_CYAN);   tft.setTextSize(3);
  tft.print("BRITE: ");
  tft.print(briteValue);
  tft.println("  ");
  delay(500);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
// Set a particular pixel to an assigned color
void setPixel(uint32_t c, uint8_t pixel, uint8_t wait) {
  strip.setPixelColor(pixel, c);
/*  Serial.print("pixel: ");
  Serial.print(pixel);
  Serial.print("  color: ");
  Serial.print(c);
  Serial.print("  wait: ");
  Serial.println(wait); */
  strip.show();
  delay(wait);
}


